"""
Achsen, vereint die Positionssensoren und die Motoren.
Steuert die Bewebung (Soll-Zielvergleich).
"""

# from logger import log
from threading import Thread
import time
import skyfield.units
import database
import interface
import motor_stepper
import motor_analog
import sensor


class Achse(Thread):
    ACHSE_AZ = interface.Interface.ACHSE_AZ
    ACHSE_EL = interface.Interface.ACHSE_EL

    def __init__(self, achse, db, interface, sensor2angle):
        Thread.__init__(self)
        assert achse in (interface.ACHSE_EL, interface.ACHSE_AZ), "'" + achse + "' ist Keine gültige Achse!"

        self.__sensor = sensor.Sensor(achse, db, interface, sensor2angle)

        # Motor
        mt = db.get_achsparameter_str(achse, "motortyp")
        if mt == interface.MOTORTYP_STEPPER:
            self.__motor = motor_stepper.MotorStepper(achse, db, interface)
        elif mt == interface.MOTORTYP_ANALOG:
            self.__motor = motor_analog.MotorAnalog(achse, db, interface)
        else:
            raise ValueError(achse + ": Kein gültiger Motortyp!")

        # Grenzen
        self.__grenze_plus = db.get_achsparameter_float(achse, "grenze+")
        self.__grenze_minus = db.get_achsparameter_float(achse, "grenze-")
        self.grenzverletzung = False
        # Schwellen
        self.__schwelle_rate = db.get_achsparameter_float(achse, "schwellerate")
        self.__schwelle_ziel = db.get_achsparameter_float(achse, "schwelleziel")

        # Thread starten
        self.__moveTo = False
        self.__destinationAngle = skyfield.units.Angle(degrees=0)
        self.__threadIsRunning = True
        self.start()

    def __del__(self):
        self.close()

    def close(self):
        if self.__threadIsRunning:
            self.__threadIsRunning = False
            self.join(1)
        self.__sensor.close()
        self.__motor.close()

    def move_to_target(self, angle):
        self.__destinationAngle = angle
        self.__moveTo = True

    def set_rate_manuell(self, rate):
        self.__moveTo = False
        self.__motor.set_rate(rate)

    def set_direction_manuell(self, dir):
        self.__moveTo = False
        self.__motor.set_direction(dir)

    def stop(self):
        self.__moveTo = False
        self.__motor.set_rate(self.__motor.MOTOR_STOP)

    @property
    def motortyp(self):
        return self.__motor.motortyp

    @property
    def motor_ison(self):
        return self.__motor.is_on

    @property
    def motor_direction_ist(self):
        return self.__motor.direction_ist

    @property
    def motor_rate_ist(self):
        return self.__motor.rate_ist

    @property
    def motor_rate_soll(self):
        return self.__motor.rate_soll

    # Sensor -------------------------
    @property
    def position(self):
        return self.__sensor.angle

    @property
    def sensorvalue(self):
        return self.__sensor.value

    @property
    def sensor_issynchronized(self):
        return self.__sensor.is_synchronized

    # Thread -----------------------
    def run(self):
        while self.__threadIsRunning:
            # Leserate des Sensors setzen
            self.__sensor.set_motor_direction(self.__motor.direction_ist)
            # Zeit die der Motor blockiert weiterleiten
            self.__motor.set_motor_is_blocked_time(self.__sensor.sensorMotorBlockedTime)

            time.sleep(0.05)   # Änderung 13.7.2019 schnellere Reaktion

            # Automatisch zu Ziel bewegen
            if self.__moveTo:
                zieldistanz = (self.__destinationAngle.degrees - self.__sensor.angle.degrees)

                if self.__destinationAngle.degrees > self.__sensor.angle.degrees:   # Ziel oberhalb anfahren
                    if self.__sensor.angle.degrees <= self.__grenze_plus:   # obere Grenze prüfen
                        self.grenzverletzung = False
                        self.__motor.set_direction(self.__motor.MOTOR_RIGHT)
                    else:
                        self.grenzverletzung = True
                        zieldistanz = 0

                elif self.__destinationAngle.degrees < self.__sensor.angle.degrees:   # Ziel unterhalb anfahren
                    if self.__sensor.angle.degrees >= self.__grenze_minus:   # untere Grenze prüfen
                        self.grenzverletzung = False
                        self.__motor.set_direction(self.__motor.MOTOR_LEFT)
                    else:
                        self.grenzverletzung = True
                        zieldistanz = 0

                if abs(zieldistanz) > self.__schwelle_rate:
                    self.__motor.set_rate(self.__motor.MOTOR_FAST)
                elif abs(zieldistanz) <= self.__schwelle_ziel:
                    self.__motor.set_rate(self.__motor.MOTOR_STOP)
                else:
                    self.__motor.set_rate(self.__motor.MOTOR_SLOW)

            # manuell bewegen -> Grenze prüfen
            else:
                if self.__sensor.is_synchronized:  # Grenzwertprüfung nur bei synchronisiertem Sensor
                    if self.__motor.direction_ist > 0:  # Bewegung nach rechts
                        if self.__sensor.angle.degrees <= self.__grenze_plus:  # obere Grenze prüfen
                            self.grenzverletzung = False
                        else:
                            self.grenzverletzung = True
                            self.stop()

                    if self.__motor.direction_ist < 0:  # Bewegung nach links
                        if self.__sensor.angle.degrees >= self.__grenze_minus:   # untere Grenze prüfen
                            self.grenzverletzung = False
                        else:
                            self.grenzverletzung = True
                            self.stop()


if __name__ == '__main__':
    import configuration
    import motor

    db = database.Database()
    config = configuration.Configuration(db)
    iface = interface.Interface(config)

    for i in ('E'):
        a = Achse(i, db, iface)
        print("Achse:", i)
        print("Motortyp:", a.motortyp )

        if True:
            a.move_to_target(60)
            time.sleep(1)
            print("MoveTo: 60;  POS:", a.position)
            time.sleep(5)
            a.move_to_target(50.1)
            print("MoveTo: 50.1;  POS:", a.position)
            time.sleep(5)
            a.move_to_target(40)
            print("MoveTo: 40;  POS:", a.position)
            time.sleep(5)
            a.move_to_target(30)
            print("MoveTo: 30;  POS:", a.position)
            time.sleep(5)
            a.move_to_target(44.6)
            print("MoveTo: 44.6;  POS:", a.position)
            time.sleep(5)
            print("END")

        else:
            a.set_direction_manuell(motor.Motor.MOTOR_LEFT)
            a.set_rate_manuell(motor.Motor.MOTOR_FAST)
            time.sleep(3)
            a.set_direction_manuell(motor.Motor.MOTOR_RIGHT)
            time.sleep(3)
            a.set_rate_manuell(motor.Motor.MOTOR_SLOW)
            time.sleep(3)
            a.set_rate_manuell(motor.Motor.MOTOR_STOP)

    a.close()
    iface.close()
    config.close()
    db.close()

