"""
Verbindung zu ArduinoNano-Interface
"""

from logger import log
import configuration
import serial
import time
import threading


class InterfaceError(Exception):
    def __init__(self, value):
        self.value = value

    def __str__(self):
        return repr(self.value)


class Interface(object):
    ACHSE_AZ = 'A'
    ACHSE_EL = 'E'

    SENSORTYP_IMPULS = 'I'
    SENSORTYP_HH12 = 'H'
    SENSORTYP_SIMULATOR = 's'

    MOTORTYP_STEPPER = 'S'
    MOTORTYP_ANALOG = 'A'
    MOTORDREHRICHTUNG_DIREKT = 'd'
    MOTORDREHRICHTUNG_INVERS = 'i'

    INTERFACEVERSION = "0.1"

    lock = threading.Lock()

    def __init__(self, config):
        self.__interface = serial.Serial()
        self.__interface.port = config.interfaceport
        self.__interface.baudrate = 9600
        self.__interface.bytesize = serial.EIGHTBITS
        self.__interface.parity = serial.PARITY_NONE
        self.__interface.stopbits = serial.STOPBITS_ONE
        self.__interface.timeout = 0.001

        self.__connect()
        self.__checkversion()

    def __del__(self):
        self.close()

    def close(self):
        if self.__interface.is_open:
            self.__interface.close()

    def __checkachse(self, achse):
        assert achse in (self.ACHSE_AZ, self.ACHSE_EL, 'd'), "Ungültige Achse: " + str(achse)   # d = für Debug

    def __checkversion(self):
        ifaceversion = self.version
        if ifaceversion != self.INTERFACEVERSION:
            x = "Benötigt Interfaceversion: " + self.INTERFACEVERSION\
                + " - Verfügbare Interfaceversion: " + ifaceversion
            log.error(x)
            raise InterfaceError(x)

    def __connect(self):
        if not self.__interface.is_open:
            try:
                log.debug("interface.open")
                self.__interface.open()
                time.sleep(1.5)  # Der Arduino brauch etwas bis er sich meldet.
            except serial.SerialException:
                x = "Verbindung zum Interface " + self.__interface.port + " kann nicht hergestellt werden!"
                log.error(x)
                raise InterfaceError(x)

    def __send(self, cmd):
        with self.lock:
            try:
                # log.debug("CMD ->: " + str(cmd) + ";")
                self.__interface.write(cmd.encode("ascii") + b';')

                answer = []
                timeoutcounter = 50
                while timeoutcounter > 0:
                    c = self.__interface.read().decode("ascii")
                    if c != "":             # Zeichen gelesen
                        answer.append(c)    # Zeichen speichern
                        if c == ";":        # Abschlusszeichen -> Schleife beenden
                            timeoutcounter = 0
                    else:
                        if timeoutcounter <= 0:  # Nix empfangen -> Timeout
                            log.error("Serial Timeout - Abschlusszeichen ';' nicht erhalten: "
                                      + str(cmd) + " ".join(answer))
                            raise serial.serialutil.SerialException
                        else:                    # kein Zeichen empfangen -> weiter auf Zeichen warten / Timeoutzähler++
                            timeoutcounter -= 1

                r = "".join(answer)
                # log.debug("ANS <-: " + r)
                return r.rstrip(";\n\r")

            except serial.serialutil.SerialException:
                log.error("Fehler in der Datenübertragung! (SerialException): " + str(cmd))

    @property
    def version(self):
        return self.__send("V")

    def set_sensortyp(self, achse, typ):
        self.__checkachse(achse)

        if typ not in (self.SENSORTYP_IMPULS, self.SENSORTYP_HH12, self.SENSORTYP_SIMULATOR):
            errortext = "Ungültiger Sensortyp: " + str(typ)
            log.error(errortext)
            raise ValueError(errortext)

        cmd = "S" + achse + "T" + typ
        self.__send(cmd)

    def set_sensorvalue(self, achse, sensorwert):
        self.__checkachse(achse)
        cmd = "S" + achse + "S" + str(sensorwert)
        self.__send(cmd)

    def set_sensorsyncvalue(self, achse, syncwert):
        self.__checkachse(achse)
        cmd = "S" + achse + "I" + str(syncwert)
        self.__send(cmd)

    def set_sensorforcesync(self, achse):
        self.__checkachse(achse)
        cmd = "S" + achse + "F"
        self.__send(cmd)

    def set_motortyp(self, achse, typ, drehrichtunspin_invertieren):
        self.__checkachse(achse)

        if typ not in (self.MOTORTYP_ANALOG, self.MOTORTYP_STEPPER):
            errortext = "Ungültiger Motortyp: " + str(typ)
            log.error(errortext)
            raise ValueError(errortext)
        if drehrichtunspin_invertieren not in (self.MOTORDREHRICHTUNG_DIREKT, self.MOTORDREHRICHTUNG_INVERS):
            errortext = "Drehrichtungspin invertieren: " + str(drehrichtunspin_invertieren)
            log.error(errortext)
            raise ValueError(errortext)

        cmd = "M" + achse + "T" + typ + drehrichtunspin_invertieren
        self.__send(cmd)

    def set_motorpower(self, achse, poweron):
        self.__checkachse(achse)
        p = "1" if poweron else "0"
        cmd = "M" + achse + "P" + p
        self.__send(cmd)

    def set_motorspeed(self, achse, speed):
        self.__checkachse(achse)
        s = "1" if speed else "0"
        cmd = "M" + achse + "S" + s
        self.__send(cmd)

    def set_motordirection(self, achse, direction):
        self.__checkachse(achse)
        d = "1" if direction else "0"
        cmd = "M" + achse + "D" + d
        self.__send(cmd)
        # TODO für Motoransteuerung zur Sicherheit immer alle Parameter übertragen. (AUS-EIN, Richtung, Rate)

    def set_steppermotorspeed(self, achse, speed):
        self.__checkachse(achse)
        if speed > 10000:
            errortext = "Schrittmotor Geschwindigkeit zu hoch: " + str(speed)
            log.error(errortext)
            raise ValueError(errortext)

        if speed == 0:
            s = int(0xffff)
        else:
            s = round(50000 / speed)

        cmd = "M" + achse + "C" + str(s)
        self.__send(cmd)

    def get_sensorvalue(self, achse):
        self.__checkachse(achse)
        return self.__send("G" + achse)


if __name__ == '__main__':
    import database

    db = database.Database()
    config = configuration.Configuration(db)
    interface = Interface(config)

    v = interface.version
    print("Interfaceversion:", v)

    interface.set_sensortyp(interface.ACHSE_AZ, interface.SENSORTYP_IMPULS)
    interface.set_sensortyp(interface.ACHSE_EL, interface.SENSORTYP_IMPULS)
    interface.set_motortyp(interface.ACHSE_AZ, interface.MOTORTYP_STEPPER, 'd')
    interface.set_motortyp(interface.ACHSE_EL, interface.MOTORTYP_STEPPER, 'i')

    interface.set_sensorvalue(interface.ACHSE_AZ, 100)
    interface.set_sensorvalue(interface.ACHSE_EL, -123100)
    interface.set_sensorsyncvalue(interface.ACHSE_AZ, 1100)
    interface.set_sensorsyncvalue(interface.ACHSE_EL, -2200)

    interface.set_motorpower(interface.ACHSE_AZ, True)
    interface.set_motorspeed(interface.ACHSE_EL, True)
    interface.set_motordirection(interface.ACHSE_AZ, True)

    print("AZ-Value: ", interface.get_sensorvalue(interface.ACHSE_AZ))


    sv = interface.get_sensorvalue('d')
    print("Debug-Value: ", sv)

    try:
        x = 'asdf' + str(0xff)
        xx = int(x[:-1])
        print(xx)
    except Exception as e:
        log.error(e)
        log.error("GET-Error: " + str(x))


    for i in range(1, 10):
        sv = interface.get_sensorvalue(interface.ACHSE_AZ)
        #print("Debug-Value: ", sv)
        if sv is not None:
            _v = int(sv[:-1])
            _s = (sv[-1] == "s")
            #print(_v, _s)

        sv = interface.get_sensorvalue(interface.ACHSE_EL)
        #print("Debug-Value: ", sv)
        if sv is not None:
            _v = int(sv[:-1])
            _s = (sv[-1] == "s")
            #print(_v, _s)



    #print("EL-Value: ", interface.get_sensorvalue(interface.ACHSE_EL))
    #time.sleep(5)
    #print("AZ-Value: ", interface.get_sensorvalue(interface.ACHSE_AZ))
    #print("EL-Value: ", interface.get_sensorvalue(interface.ACHSE_EL))

    interface.close()
