"""
Motoren der Achsen
"""

from logger import log
import database
import interface
from abc import ABC, abstractmethod


class Motor(ABC):
    MOTOR_LEFT = -1
    MOTOR_RIGHT = 1
    MOTOR_FAST = 2
    MOTOR_SLOW = 1
    MOTOR_STOP = 0

    MOTORTYP_STEPPER = interface.Interface.MOTORTYP_STEPPER
    MOTORTYP_ANALOG = interface.Interface.MOTORTYP_ANALOG

    def __init__(self, achse, db, interface):
        assert achse in (interface.ACHSE_EL, interface.ACHSE_AZ), "'" + achse + "' ist keine gültige Achse!"

        self._db = db
        self._achse = achse
        self._interface = interface
        self.motortyp = "?"

    def __del__(self):
        self.close()

    @abstractmethod
    def close(self):
        pass

    @property
    @abstractmethod
    def is_on(self):
        pass

    @property
    @abstractmethod
    def direction_ist(self):
        pass

    @property
    @abstractmethod
    def rate_ist(self):
        pass

    @property
    @abstractmethod
    def rate_soll(self):
        pass

    def _set_motortype2interface(self):
        _mt = self._db.get_achsparameter_str(self._achse, "motortyp")
        if _mt != self.motortyp:
            raise AttributeError("Motortyp mismatch! Achse:" + self._achse +
                                 " Anwendung:" + self.motortyp + " Parameter:" + _mt)

        _drechrichtunginvers = self._db.get_achsparameter_str(self._achse, "motorrichtunginvers")

        self._interface.set_motortyp(self._achse, self.motortyp, _drechrichtunginvers)

    @abstractmethod
    def set_direction(self, direction):
        if direction not in (self.MOTOR_LEFT, self.MOTOR_RIGHT):
            errortext = "Ungültige Motorrichtung: " + str(direction)
            log.error(errortext)
            raise ValueError(errortext)

    @abstractmethod
    def set_rate(self, rate):
        if rate not in (self.MOTOR_FAST, self.MOTOR_SLOW, self.MOTOR_STOP):
            errortext = "Ungültige Motorgeschwindigkeit: " + str(rate)
            log.error(errortext)
            raise ValueError(errortext)

    @abstractmethod
    def set_motor_is_blocked_time(self, blockedtime):
        """Zeit (sec) die der Motor blockiert ist."""
        pass

    @abstractmethod
    def emergency_stop(self):
        pass


if __name__ == '__main__':
    import configuration
    import motor_stepper
    import motor_analog
    import time

    db = database.Database()
    config = configuration.Configuration(db)
    iface = interface.Interface(config)
    achse = 'A'

    mt = db.get_achsparameter_str(achse, "motortyp")
    motor = None
    if mt == iface.MOTORTYP_STEPPER:
        motor = motor_stepper.MotorStepper(achse, db, iface)
    else:
        motor = motor_analog.MotorAnalog(achse, db, iface)


    print("Achse:", achse)
    print("Motortyp:", motor.motortyp)

    #print("SensorN:", a.ttt)
    #time.sleep(3)
    #print("SensorS:", a.ttt)

    motor.set_direction(motor.MOTOR_LEFT)
    motor.set_rate(motor.MOTOR_FAST)

    time.sleep(4)
    motor.set_direction(motor.MOTOR_RIGHT)
    time.sleep(8)


    motor.close()
    iface.close()
    config.close()
    db.close()

