"""
Positionssensoren (Winkelmesser) der Achsen
"""

from logger import log
import database
import interface
import sensor_toangle
from threading import Thread
import time


class Sensor(Thread):
    SENSORTYP_IMPULS = interface.Interface.SENSORTYP_IMPULS
    SENSORTYP_HH12 = interface.Interface.SENSORTYP_HH12
    SENSORTYP_SIMULATOR = interface.Interface.SENSORTYP_SIMULATOR

    def __init__(self, achse, db, interface, sensor2angle):
        Thread.__init__(self)
        assert achse in (interface.ACHSE_EL, interface.ACHSE_AZ), "'" + achse + "' ist Keine gültige Achse!"

        self.__db = db
        self.__achse = achse
        self.__interface = interface
        self.__sensor2angle = sensor2angle

        # Sensor
        self.__sensortyp = "?"
        self.__sensorBlockedMotorWaitToSync = 9999999999
        self.sensorMotorBlockedTime = 0
        self.__sensorValue = 0
        self.__sensorIsSyncronized = False
        self.__sensorInitValue = 0
        self.__sensor_motorDirection = 0

        self.__load_parameter()
        self.__threadIsRunning = True
        self.__sensorReadIntervall = 1
        self.__sensorReadIntervallDelay = 0
        self.start()


    def __del__(self):
        self.close()


    def close(self):
        if self.__threadIsRunning:
            self.__threadIsRunning = False
            self.join(self.__sensorReadIntervall + 5)


    def __load_parameter(self):
        self.__sensortyp = self.__db.get_achsparameter_str(self.__achse, "sensortyp")
        self.__interface.set_sensortyp(self.__achse, self.__sensortyp)

        # Impulssensor oder Simulator
        if self.__sensortyp in (self.SENSORTYP_IMPULS, self.SENSORTYP_SIMULATOR):
            self.__sensorBlockedMotorWaitToSync = self.__db.get_achsparameter_int(self.__achse, "sensorsyncdelay")
            i = self.__db.get_achsparameter_int(self.__achse, "sensorinit")
            s = self.__db.get_achsparameter_int(self.__achse, "sensorsync")
            self.__interface.set_sensorvalue(self.__achse, i)
            self.__interface.set_sensorsyncvalue(self.__achse, s)
            self.__sensorInitValue = i


    def set_motor_direction(self, direction):
        self.__sensor_motorDirection = direction

        # Leserate des Sensors abhängig davon setzen ob der Motor aktiv ist.
        if direction != 0:
            self.__sensorReadIntervall = 1
            if self.__sensorReadIntervallDelay > self.__sensorReadIntervall:
                self.__sensorReadIntervallDelay = self.__sensorReadIntervall
        else:
            self.__sensorReadIntervall = 13


    @property
    def typ(self):
        return self.__sensortyp


    @property
    def value(self):
        return self.__sensorValue


    @property
    def angle(self):
        # Solange nicht synchronisiert ist, immer den Winkel des Initialwertes liefern
        v = self.__sensorValue if self.__sensorIsSyncronized else self.__sensorInitValue

        if self.__achse == self.__interface.ACHSE_AZ:
            return self.__sensor2angle.get_az_angle(v)
        else:
            return self.__sensor2angle.get_el_angle(v)


    @property
    def is_synchronized(self):
        return self.__sensorIsSyncronized

    
    # Thread -----------------------
    def run(self):
        impulssensorsyncoldvalue = 0
        lastimpulsetime = time.time()

        while self.__threadIsRunning:
            time.sleep(0.05)
            self.__sensorReadIntervallDelay -= 1

            if self.__sensorReadIntervallDelay <= 0:
                self.__sensorReadIntervallDelay = self.__sensorReadIntervall

                sv = self.__interface.get_sensorvalue(self.__achse)
                if sv is not None:
                    try:
                        self.__sensorValue = int(sv[:-1])
                        self.__sensorIsSyncronized = (sv[-1] == "s")
                    except Exception as e:
                        log.error(e)
                        log.error("GET-" + self.__achse + " Lesefehler: " + str(sv))

            # Impulssensor am "Endanschlag" synchronisieren
            if self.__sensortyp in (self.SENSORTYP_IMPULS, self.SENSORTYP_SIMULATOR) and self.__sensor_motorDirection != 0:
                if impulssensorsyncoldvalue == self.__sensorValue:
                    self.sensorMotorBlockedTime = time.time() - lastimpulsetime
                    # Motor ist ein, aber es gab für definierte Zeit keine Änderung des Sensorwertes
                    # if impulssensorsyncdelay <= 0:
                    if self.sensorMotorBlockedTime > self.__sensorBlockedMotorWaitToSync:
                        if self.__sensor_motorDirection < 0:        # Unterer Syncpunkt
                            s = self.__db.get_achsparameter_int(self.__achse, "sensorsync-")
                            self.__interface.set_sensorvalue(self.__achse, s)
                            self.__interface.set_sensorforcesync(self.__achse)
                            log.info("Unterer Syncpunkt gesetzt (" + self.__achse + "): " + str(s))
                        elif self.__sensor_motorDirection > 0:      # Oberer Syncpunkt
                            s = self.__db.get_achsparameter_int(self.__achse, "sensorsync+")
                            self.__interface.set_sensorvalue(self.__achse, s)
                            self.__interface.set_sensorforcesync(self.__achse)
                            log.info("Oberer Syncpunkt gesetzt (" + self.__achse + "): " + str(s))
                else:
                    impulssensorsyncoldvalue = self.__sensorValue
                    self.sensorMotorBlockedTime = 0
                    lastimpulsetime = time.time()
            else:
                lastimpulsetime = time.time()   # Motorblockierzeit zurücksetzen wenn Motor ohnehin steht.


if __name__ == '__main__':
    import configuration

    db = database.Database()
    config = configuration.Configuration(db)
    iface = interface.Interface(config)
    sensor2angle = sensor_toangle.Sensor2Angle(db)

    e = Sensor('E', db, iface, sensor2angle)
    print("Sensortyp:", e.typ)
    a = Sensor('A', db, iface, sensor2angle)
    print("Sensortyp:", a.typ)
    time.sleep(1)

    print("AZ", a.value, a.angle)
    print("EL", e.value, e.angle)
    print("AZ", a.value, a.angle)
    print("EL", e.value, e.angle)

    # for i in range(1, 50):
    #    time.sleep(1)
    #    print("Sensor:", a.value, a.isSynchronized)
    #    if i == 10: a.setSensorReadIntervall(0)

    a.close()
    e.close()
    iface.close()
    config.close()
    db.close()
