"""
Werte der Positionssensoren der Achsen in Winkel übersetzen.
"""

from logger import log
import database
import skyfield.api
import skyfield.units


class Sensor2Angle(object):
    def __init__(self, db):
        self.__db = db
        self.__sensorValueAZ = 0
        self.__sensorValueEL = 0
        self.__lastAngleAZ = skyfield.units.Angle(degrees=0)
        self.__lastAngleEL = skyfield.units.Angle(degrees=0)

        # Enden der berechneten Transfomationsmatrix ermitteln
        self.__minAZ = None
        self.__maxAZ = None
        self.__minEL = None
        self.__maxEL = None
        self.__read_minmax()

    def __del__(self):
        self.close()

    def close(self):
        pass

    def get_az_angle(self, sensorvalue):
        """Aus einem AZ-Sensorwert den zugehörigen Winkel lesen.
           Für unbekannte Sensorwerte wird jeweils der größte bzw. der kleinste Winkel geliefert."""
        if self.__sensorValueAZ != sensorvalue:
            self.__sensorValueAZ = sensorvalue
            cur = self.__db.sl3database.cursor()
            cur.execute('select winkel from transform_az where sensorvalue = ?', (sensorvalue, ))
            data = cur.fetchone()
            if data is not None:
                w = float(data[0])
                self.__lastAngleAZ = skyfield.units.Angle(degrees=w)
            else:   # Fehler im Sensorwert
                log.error("Sensorwert (" + str(sensorvalue) + ") ausserhalb des bekannten Bereiches!")
                if sensorvalue < self.__minAZ[0]:
                    self.__lastAngleAZ = skyfield.units.Angle(degrees=self.__minAZ[1])
                elif sensorvalue > self.__maxAZ[0]:
                    self.__lastAngleAZ = skyfield.units.Angle(degrees=self.__maxAZ[1])
                else:
                    raise ValueError("Sensorwert nicht bekannt!")
        return self.__lastAngleAZ

    def get_el_angle(self, sensorvalue):
        """Aus einem EL-Sensorwert den zugehörigen Winkel lesen.
           Für unbekannte Sensorwerte wird jeweils der größte bzw. der kleinste Winkel geliefert."""
        if self.__sensorValueEL != sensorvalue:
            self.__sensorValueEL = sensorvalue
            cur = self.__db.sl3database.cursor()
            cur.execute('select winkel from transform_el where sensorvalue = ?', (sensorvalue, ))
            data = cur.fetchone()
            if data is not None:
                w = float(data[0])
                self.__lastAngleEL = skyfield.units.Angle(degrees=w)
            else:   # Fehler im Sensorwert
                log.error("Sensorwert (" + str(sensorvalue) + ") ausserhalb des bekannten Bereiches!")
                if sensorvalue < self.__minEL[0]:
                    self.__lastAngleEL = skyfield.units.Angle(degrees=self.__minEL[1])
                elif sensorvalue > self.__maxEL[0]:
                    self.__lastAngleEL = skyfield.units.Angle(degrees=self.__maxEL[1])
                else:
                    raise ValueError("Sensorwert nicht bekannt!")
        return self.__lastAngleEL

    def __read_minmax(self):
        """Für unbekannte Sensorwerte den jeweils größten bzw. kleinsten Winkel lesen."""
        cur = self.__db.sl3database.cursor()
        cur.execute('select sensorvalue, winkel from transform_az order by sensorvalue asc limit 1')
        self.__minAZ = cur.fetchone()
        cur.execute('select sensorvalue, winkel from transform_az order by sensorvalue desc limit 1')
        self.__maxAZ = cur.fetchone()
        cur.execute('select sensorvalue, winkel from transform_el order by sensorvalue asc limit 1')
        self.__minEL = cur.fetchone()
        cur.execute('select sensorvalue, winkel from transform_el order by sensorvalue desc limit 1')
        self.__maxEL = cur.fetchone()

    # ##################################################################################################################
    # Transformwerte AZ berechnen ---------------------------------------
    def calculate_transform_az(self):
        """Aus den AZ-Referenzpunkten werden für alle möglichen Sensorwerte die Winkelwerte errechnet."""

        cur = self.__db.sl3database.cursor()
        cur.execute('delete from transform_az')

        sql = '''select sensorvalue, winkel from referenzpunkt_az order by sensorvalue'''
        cur.execute(sql)

        i = 0
        d2 = None
        for data in cur.fetchall():
            i += 1
            if i == 1:
                d2 = data
            elif i >= 2:
                d1 = d2
                d2 = data
                self.__calculate_transform_az_interpolieren(cur, d1, d2)

        if i < 2:
            log.error("Zu wenige Referenzpunkte!")
            return False

        # letzen Referenzpunkt in Transfertabelle übertragen
        sql = '''insert into transform_az (sensorvalue, winkel) values (:sv, :wi)'''
        p = {"sv": d2[0], "wi": round(d2[1], 8)}
        cur.execute(sql, p)

        self.__calculate_transform_az_interpolieren_north(cur)
        cur.execute('reindex ix_transform_az')
        self.__db.sl3database.commit()
        return True

    @staticmethod
    def __calculate_transform_az_interpolieren(cur, d1, d2):
        log.debug("Inerpolierbereich AZ: " + str(d1) + " - " + str(d2))
        schritte = (d2[0] - d1[0])
        winkelabstand = (d2[1] - d1[1]) / schritte
        sql = '''insert into transform_az (sensorvalue, winkel) values (:sv, :wi)'''
        for i in range(schritte):
            p = {"sv": i + d1[0], "wi": round(winkelabstand * i + d1[1], 8)}
            cur.execute(sql, p)

    @staticmethod
    def __calculate_transform_az_interpolieren_north(cur):
        # den östlichsten Referenzpunkt lesen
        sql = '''select sensorvalue, winkel from transform_az order by sensorvalue asc limit 1'''
        cur.execute(sql)
        ostpunkt = cur.fetchone()
        # den westlichsten Referenzpunkt lesen
        sql = '''select sensorvalue, winkel from transform_az order by sensorvalue desc limit 1'''
        cur.execute(sql)
        westpunkt = cur.fetchone()

        sued_schritte = (westpunkt[0] - ostpunkt[0])
        sued_winkel = (westpunkt[1] - ostpunkt[1])
        winkelabstand = sued_winkel / sued_schritte

        if winkelabstand <= 0:
            log.error("Inerpolierbereich AZ-Nordlücke - Winkelfehler: " + str(winkelabstand))
            raise ValueError("Inerpolierbereich AZ-Nordlücke - Winkelfehler: " + str(winkelabstand))

        # die "Nordlücke" gleichmäßig mit Sensorwert-Winkel-Paaren füllen
        sql = '''insert into transform_az (sensorvalue, winkel) values (:sv, :wi)'''
        ost_schritt = ostpunkt[0] - 1
        ost_winkel = ostpunkt[1] - winkelabstand
        while ost_winkel >= 0:  # bis 0° (Nord) berechnen
            p = {"sv": ost_schritt, "wi": round(ost_winkel, 8)}
            cur.execute(sql, p)
            ost_schritt -= 1
            ost_winkel -= winkelabstand

        west_schritt = westpunkt[0] + 1
        west_winkel = westpunkt[1] + winkelabstand
        while west_winkel < 360:  # bis 360° (Nord) berechnen
            p = {"sv": west_schritt, "wi": round(west_winkel, 8)}
            cur.execute(sql, p)
            west_schritt += 1
            west_winkel += winkelabstand

    # Transformwerte EL berechnen ---------------------------------------
    def calculate_transform_el(self):
        """Aus den EL-Referenzpunkten werden für alle möglichen Sensorwerte die Winkelwerte errechnet."""

        cur = self.__db.sl3database.cursor()
        cur.execute('delete from transform_el')

        sql = '''select sensorvalue, winkel from referenzpunkt_el order by sensorvalue'''
        cur.execute(sql)
        i = 0
        d2 = None
        for data in cur.fetchall():
            i += 1
            if i == 1:
                d2 = data
            elif i >= 2:
                d1 = d2
                d2 = data
                self.__calculate_transform_el_interpolieren(cur, d1, d2)

        if i < 2:
            log.error("Zu wenige Referenzpunkte!")
            return False

        # letzen Referenzpunkt in Transfertabelle übertragen
        sql = '''insert into transform_el (sensorvalue, winkel) values (:sv, :wi)'''
        p = {"sv": d2[0], "wi": round(d2[1], 8)}
        cur.execute(sql, p)

        self.__calculate_transform_el_interpolieren_down(cur)
        self.__calculate_transform_el_interpolieren_up(cur)
        cur.execute('reindex ix_transform_el')
        self.__db.sl3database.commit()
        return True

    @staticmethod
    def __calculate_transform_el_interpolieren(cur, d1, d2):
        log.debug("Inerpolierbereich EL: " + str(d1) + " - " + str(d2))
        schritte = (d2[0] - d1[0])

        winkelabstand = (d2[1] - d1[1]) / schritte
        sql = '''insert into transform_el (sensorvalue, winkel) values (:sv, :wi)'''
        for i in range(schritte):
            p = {"sv": i + d1[0], "wi": round(winkelabstand * i + d1[1], 8)}
            cur.execute(sql, p)

    @staticmethod
    def __calculate_transform_el_interpolieren_down(cur):
        sql = '''select sensorvalue, winkel from transform_el order by sensorvalue asc limit 2'''
        cur.execute(sql)
        d1 = cur.fetchone()
        d2 = cur.fetchone()
        winkelabstand = (d2[1] - d1[1])
        if winkelabstand <= 0:
            log.error("Inerpolierbereich EL-down - Winkelfehler: " + str(winkelabstand))
            raise ValueError("Inerpolierbereich EL-down - Winkelfehler: " + str(winkelabstand))

        d = d1[0]
        w = d1[1]
        sql = '''insert into transform_el (sensorvalue, winkel) values (:sv, :wi)'''
        while w > -20:  # bis -20° (unterm Horizont) berechnen
            d -= 1
            w -= winkelabstand
            p = {"sv": d, "wi": round(w, 8)}
            cur.execute(sql, p)

    @staticmethod
    def __calculate_transform_el_interpolieren_up(cur):
        sql = '''select sensorvalue, winkel from transform_el order by sensorvalue desc limit 2'''
        cur.execute(sql)
        d2 = cur.fetchone()
        d1 = cur.fetchone()
        winkelabstand = (d2[1] - d1[1])
        if winkelabstand <= 0:
            log.error("Inerpolierbereich EL-doupwn - Winkelfehler: " + str(winkelabstand))
            raise ValueError("Inerpolierbereich EL-doupwn - Winkelfehler: " + str(winkelabstand))

        d = d2[0]
        w = d2[1]
        sql = '''insert into transform_el (sensorvalue, winkel) values (:sv, :wi)'''
        while w < 110:  # bis 110° (20° über Zenit) berechnen
            d += 1
            w += winkelabstand
            p = {"sv": d, "wi": round(w, 8)}
            cur.execute(sql, p)


if __name__ == '__main__':
    db = database.Database()
    sto = Sensor2Angle(db)

    #sto.calculate_transform_el()
    #sto.calculate_transform_az()

    print("AZ:", sto.get_az_angle(-18000))
    print("EL:", sto.get_el_angle(-4500))

    sto.close()
    db.close()
